#include <iostream>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <GL/gl.h>

#include "init.hpp"
#include "error_msg.h"

static HWND__ *win;
static HGLRC__ *wgl_ctx;
static HDC__ *dc;

long CALLBACK HandleEvents(HWND__ *win, unsigned int msg, unsigned int wparam, long lparam);

int InitGraphics(InitParams *iparams) {

	HINSTANCE pid = GetModuleHandle(0);

	WNDCLASSEX wc;
	memset(&wc, 0, sizeof wc);
	wc.cbSize = sizeof wc;
	wc.hbrBackground = (HBRUSH)GetStockObject(NULL_BRUSH);
	wc.hCursor = LoadCursor(0, IDC_ARROW);
	wc.hIcon = wc.hIconSm = LoadIcon(0, IDI_APPLICATION);
	wc.hInstance = pid;
	wc.lpfnWndProc = HandleEvents;
	wc.lpszClassName = "win32_sucks_big_time";
	wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	RegisterClassEx(&wc);

	unsigned long style = WS_OVERLAPPEDWINDOW | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	win = CreateWindow("win32_sucks_big_time", "Sense Amok Graphics System on Win32", 
						style, 0, 0, iparams->x, iparams->y, 0, 0, pid, 0);
	dc = GetDC(win);

	int color_bits = iparams->bpp == 15 || iparams->bpp == 16 ? 5 : 8;
	int stencil_bits = color_bits == 8 ? 8 : 1;
		
	if(iparams->zbuffer == 32) {
		iparams->zbuffer = 24;
	}

	PIXELFORMATDESCRIPTOR pfd;
	memset(&pfd, 0, sizeof pfd);
	pfd.nSize = sizeof pfd;
	pfd.nVersion = 1;
	pfd.dwFlags = PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER | PFD_DRAW_TO_WINDOW;
	pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.cColorBits = iparams->bpp;
	pfd.cDepthBits = iparams->zbuffer;
	pfd.cStencilBits = stencil_bits;
	pfd.iLayerType = PFD_MAIN_PLANE;

	info("Initializing graphics system");
	info("Requested video mode: %dx%d %d bpp, %d zbuffer, %d stencil", iparams->x, iparams->y, iparams->bpp, iparams->zbuffer, stencil_bits);

	int pix_format = ChoosePixelFormat(dc, &pfd);
	if(!pix_format) {
		error("ChoosePixelFormat() failed");
		ReleaseDC(win, dc);
		DestroyWindow(win);
		return -1;
	}

	if(!SetPixelFormat(dc, pix_format, &pfd)) {
		error("SetPixelFormat() failed");
		ReleaseDC(win, dc);
		DestroyWindow(win);
		return -1;
	}

	if(!(wgl_ctx = wglCreateContext(dc))) {
		error("wglCreateContext() failed");
		ReleaseDC(win, dc);
		DestroyWindow(win);
		return -1;
	}

	if(wglMakeCurrent(dc, wgl_ctx) == FALSE) {
        error("wglMakeCurrent() failed");
		wglDeleteContext(wgl_ctx);
		ReleaseDC(win, dc);
		DestroyWindow(win);
		return -1;
	}

	return 0;
}


void DestroyGraphics() {
	wglMakeCurrent(0, 0);
	wglDeleteContext(wgl_ctx);
	ReleaseDC(win, dc);
	DestroyWindow(win);
}


void Flip() {
	SwapBuffers(dc);
}
