#include <iostream>
#include <cmath>
#include <cstring>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <GL/gl.h>
#include "init.hpp"
#include "wm.hpp"
#include "timer.h"

ntimer timer;	// my C cross-platform timer

int Init();
void CleanUp();
void Redraw();
void KeyHandler(int key, int state);

int main() {
		
	if(Init() == -1) {
		return -1;
	}

	/* Start the event loop, exits implicitly by the window close button
	 * or explicitly by a call to ExitEventLoop()
	 */
	EventLoop();

	CleanUp();
	return 0;
}


int Init() {
	// InitGraphics() takes a InitParams struct and creates the gl context
	InitParams iparams = {640, 480, 32, 24, true, false};
	if(InitGraphics(&iparams) == -1) return -1;

	// add function pointers (callbacks) for each handler
	// if more than one is added, more than one is called for each event.
	AddKeyHandler(KeyHandler);

	// set the idle handler and the draw handler to our drawing function
	// btw if we set no idle handler it does not busy-loop, instead it
	// goes into event-based processing, sleeping until an event arrives
	AddIdleHandler(Redraw);
	AddDrawHandler(Redraw);

	timer_reset(&timer);	// initialize the timer
	return 0;
}

void CleanUp() {
	DestroyGraphics();
}

void Redraw() {
	unsigned long time = timer_getmsec(&timer);
	float t = time / 1000.0f;
	
	glClearColor(0.5 * sin(t) + 0.5, 0.0, 0.5 * -sin(t) + 0.5, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);

	Flip();	// this wraps glxSwapBuffers()
}


void KeyHandler(int key, int state) {
	if(key == KEY_ESCAPE) ExitEventLoop();
}
