#include <vector>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "wm.hpp"

using std::vector;

static bool process_events = true;
static bool busy_loop = false;

static vector<void (*)(int, int)> key_handlers;
static vector<void (*)(int, int, int, int, int)> mouse_handlers;
static vector<void (*)()> draw_handlers;
static vector<void (*)()> idle_handlers;

long CALLBACK HandleEvents(HWND__ *win, unsigned int msg, unsigned int wparam, long lparam);
int VKeyToXKeySym(unsigned int vkey);

void AddKeyHandler(void (*handler)(int, int)) {
	key_handlers.push_back(handler);
}

void AddMouseHandler(void (*handler)(int, int, int, int, int)) {
	mouse_handlers.push_back(handler);
}

void AddDrawHandler(void (*handler)()) {
	draw_handlers.push_back(handler);
}

void AddIdleHandler(void (*handler)()) {
	idle_handlers.push_back(handler);
	busy_loop = true;
}

int EventLoop() {

	while(process_events) {

		if(busy_loop) {
			MSG event;
			while(PeekMessage(&event, 0, 0, 0, PM_REMOVE)) {
				TranslateMessage(&event);
				DispatchMessage(&event);
			}
			
			int idle_handler_count = idle_handlers.size();
			for(int i=0; i<idle_handler_count; i++) {
				idle_handlers[i]();
			}
		} else {
			MSG event;
			if(!GetMessage(&event, 0, 0, 0)) ExitEventLoop();
			TranslateMessage(&event);
			DispatchMessage(&event);
		}
	}

	return 0;
}

void ExitEventLoop() {
	process_events = false;
}

long CALLBACK HandleEvents(HWND__ *win, unsigned int msg, unsigned int wparam, long lparam) {
	static int window_mapped;

	switch(msg) {
	case WM_SHOWWINDOW:
		if(wparam) {
			window_mapped = 1;
		} else {
            window_mapped = 0;
		}
		break;

	case WM_PAINT:
		if(window_mapped) {
			int draw_handler_count = draw_handlers.size();
			for(int i=0; i<draw_handler_count; i++) {
				draw_handlers[i]();
			}
		}
		break;

	case WM_CLOSE:
		ExitEventLoop();
		break;

	case WM_KEYDOWN:
		{
			int key_handler_count = key_handlers.size();
			for(int i=0; i<key_handler_count; i++) {
				key_handlers[i](VKeyToXKeySym(wparam), 1);
			}
		}
		break;

	default:
		break;
	}

	return DefWindowProc(win, msg, wparam, lparam);;
}

int VKeyToXKeySym(unsigned int vkey) {
	return 0xff << 8 | (vkey & 0xff);
}