#include <vector>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include "wm.hpp"

using std::vector;

static bool process_events = true;
static bool busy_loop = false;
extern Display *dpy;

static vector<void (*)(int, int)> key_handlers;
static vector<void (*)(int, int, int, int, int)> mouse_handlers;
static vector<void (*)()> draw_handlers;
static vector<void (*)()> idle_handlers;


static int HandleEvents(XEvent &event);


void AddKeyHandler(void (*handler)(int, int)) {
	key_handlers.push_back(handler);
}

void AddMouseHandler(void (*handler)(int, int, int, int, int)) {
	mouse_handlers.push_back(handler);
}

void AddDrawHandler(void (*handler)()) {
	draw_handlers.push_back(handler);
}

void AddIdleHandler(void (*handler)()) {
	idle_handlers.push_back(handler);
	busy_loop = true;
}

int EventLoop() {

	while(process_events) {

		if(busy_loop) {
			while(XPending(dpy)) {
				XEvent event;
				XNextEvent(dpy, &event);
				if(HandleEvents(event) == -1) ExitEventLoop();
			}
			
			int idle_handler_count = idle_handlers.size();
			for(int i=0; i<idle_handler_count; i++) {
				idle_handlers[i]();
			}
		} else {

			XEvent event;
			XNextEvent(dpy, &event);
			if(HandleEvents(event) == -1) ExitEventLoop();
		}
	}

	return 0;
}

void ExitEventLoop() {
	process_events = false;
}

static int HandleEvents(XEvent &event) {
	static int window_mapped;

	switch(event.type) {
	case MapNotify:
		window_mapped = 1;
		break;

	case UnmapNotify:
		window_mapped = 0;
		break;

	case Expose:
		if(window_mapped && event.xexpose.count == 0) {
			int draw_handler_count = draw_handlers.size();
			for(int i=0; i<draw_handler_count; i++) {
				draw_handlers[i]();
			}
		}
		break;

	case ClientMessage:
		{
			char *atom_name = XGetAtomName(dpy, event.xclient.message_type);
			if(!strcmp(atom_name, "WM_PROTOCOLS")) {
				XFree(atom_name);
				return -1;
			}
			XFree(atom_name);
		}
		break;

	case KeyPress:
		{
			KeySym keysym = XLookupKeysym(&event.xkey, 0);
			//key_state[keysym] = 1;

			int key_handler_count = key_handlers.size();
			for(int i=0; i<key_handler_count; i++) {
				key_handlers[i](keysym, 1);
			}
		}
		break;

	default:
		break;
	}

	return 0;
}
